/*
 * lftp and utils
 *
 * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>
#include <assert.h>
#include "MirrorJob.h"
#include "rmJob.h"
#include "mkdirJob.h"
#include "PutJob.h"
#include "misc.h"
#include "xalloca.h"

void  MirrorJob::PrintStatus(int v)
{
   char *tab="\t";

   if(v!=-1)
      SessionJob::PrintStatus(v);
   else
      tab="";

   if(!Done())
      return;
   printf("%sTotal: %d director%s, %d file%s, %d symlink%s\n",tab,
      dirs,dirs==1?"y":"ies",
      tot_files,tot_files==1?"":"s",
      tot_symlinks,tot_symlinks==1?"":"s");
   if(new_files || new_symlinks)
      printf("%sNew: %d file%s, %d symlink%s\n",tab,
	 new_files,new_files==1?"":"s",
	 new_symlinks,new_symlinks==1?"":"s");
   if(mod_files || mod_symlinks)
      printf("%sModified: %d file%s, %d symlink%s\n",tab,
	 mod_files,mod_files==1?"":"s",
	 mod_symlinks,mod_symlinks==1?"":"s");
   if(del_dirs || del_files || del_symlinks)
      printf("%s%semoved: %d director%s, %d file%s, %d symlink%s\n",tab,
	 flags&DELETE?"R":"To be r",
	 del_dirs,del_dirs==1?"y":"ies",
	 del_files,del_files==1?"":"s",
	 del_symlinks,del_symlinks==1?"":"s");
}

void  MirrorJob::ShowRunStatus(StatusLine *s)
{
   switch(state)
   {
   case(INITIAL_STATE):
   case(DONE):
      s->Show("");
      break;

   case(WAITING_FOR_SUBGET):
   case(WAITING_FOR_SUBMIRROR):
   case(WAITING_FOR_RM_BEFORE_PUT):
   case(WAITING_FOR_MKDIR_BEFORE_SUBMIRROR):
      if(waiting && !waiting->Done())
	 waiting->ShowRunStatus(s);
      break;

   case(MAKE_REMOTE_DIR):
      s->Show("mkdir `%s' [%s]",remote_dir,session->CurrentStatus());
      break;

   case(CHANGING_REMOTE_DIR):
      s->Show("CWD  `%s' [%s]",remote_dir,session->CurrentStatus());
      break;

   case(GETTING_SHORT_LIST):
      s->Show("NLST `%s' [%s]",remote_dir,session->CurrentStatus());
      break;

   case(GETTING_LONG_LIST):
      s->Show("LIST `%s' [%s]",remote_dir,session->CurrentStatus());
      break;

   case(GETTING_INFO):
      if(session->IsOpen())
	 s->Show("Getting files info [%s]",session->CurrentStatus());
      break;
   }
}

void  MirrorJob::HandleFile(int how)
{
   ArgV *args;
   int	 res;
   mode_t mode;
   GetJob   *gj;
   struct stat st;

   // dir_name returns pointer to static data - need to dup it.
   char *local_name=dir_file(local_dir,file->name);
   local_name=alloca_strdup(local_name);
   char *remote_name=dir_file(remote_dir,file->name);

#if 0
   if(!(rx_include && regexec(&rxc_include,remote_name,0,0,REG_NOSUB)==0)
    && ((rx_exclude && regexec(&rxc_exclude,remote_name,0,0,REG_NOSUB)==0)
        || (rx_include && !rx_exclude)))
   {
      to_transfer->next();
      return;
   }
#endif

   if(file->defined&file->TYPE)
   {
      switch(file->filetype)
      {
      case(file->NORMAL):
      {
      try_get:
	 bool cont_this=false;
	 if(how!=0)
	 {
	    to_transfer->next();
	    break;
	 }
	 if(flags&REVERSE)
	 {
	    if(remote_set->FindByName(file->name)!=0)
	    {
	       args=new ArgV("rm");
	       args->Append(file->name);
	       waiting=new rmJob(Clone(),args);
	       waiting->parent=this;
	       waiting->cmdline=args->Combine();
	       mod_files++;
	    }
	    else
	       new_files++;
	    state=WAITING_FOR_RM_BEFORE_PUT;
	    break;
	 }
	 if(lstat(local_name,&st)!=-1)
	 {
	    if((flags&CONTINUE) && S_ISREG(st.st_mode)
	    && (file->defined&(file->DATE|file->DATE_UNPREC))
	    && file->date + prec < st.st_mtime
	    && (file->defined&file->SIZE) && file->size >= st.st_size)
	       cont_this=true;
	    else
	       remove(local_name);
	    mod_files++;
	 }
	 else
	 {
	    new_files++;
	 }
	 // launch get job
	 args=new ArgV("get");
	 args->Append(file->name);
	 args->Append(local_name);
	 gj=new GetJob(Clone(),args,cont_this);
	 if(file->defined&(file->DATE|file->DATE_UNPREC))
	    gj->SetTime(file->date);
	 waiting=gj;
	 waiting->parent=this;
	 waiting->cmdline=(char*)xmalloc(10+strlen(file->name));
	 sprintf(waiting->cmdline,"\\get %s",file->name);
	 break;
      }
      case(file->DIRECTORY):
      {
	 if(how!=1 || (flags&NO_RECURSION))
	 {
	    to_transfer->next();
	    break;
	 }
	 if(!strcmp(file->name,".") || !strcmp(file->name,".."))
	 {
	    to_transfer->next();
	    break;
	 }
	 if(flags&REVERSE)
	 {
	    if(!dir_made)
	    {
	       FileInfo *f=remote_set->FindByName(file->name);
	       if(f==0)
	       {
		  args=new ArgV("mkdir");
		  args->Append("--");
		  args->Append(file->name);
		  waiting=new mkdirJob(Clone(),args);
		  waiting->parent=this;
		  waiting->cmdline=args->Combine();
	       }
	       else
		  dir_made=true;
	       state=WAITING_FOR_MKDIR_BEFORE_SUBMIRROR;
	       break;
	    }
	 }
	 else // !REVERSE
	 {
	    mode=((file->defined&file->MODE)&&!(flags&NO_PERMS))?file->mode:0755;
	    res=mkdir(local_name,mode|0700);
	    if(res==-1 && errno==EEXIST)
	    {
	       struct stat st;
	       if(stat(local_name,&st)!=-1)
	       {
		  if(S_ISDIR(st.st_mode))
		     chmod(local_name,st.st_mode|0700);
	    	  else
		  {
		     remove(local_name);
		     res=mkdir(local_name,mode|0700);
		  }
	       }
	    }
	 }
	 // launch sub-mirror
	 MirrorJob *mj=new MirrorJob(Clone(),local_name,remote_name);
	 waiting=mj;
	 waiting->parent=this;
	 waiting->cmdline=(char*)xmalloc(strlen("\\mirror")+1+
					 strlen(file->name)+1);
	 sprintf(waiting->cmdline,"\\mirror %s",file->name);

	 // inherit flags and other things
	 mj->SetFlags(flags,1);
	 mj->SetPrec(prec);
	 if(rx_include)	mj->SetInclude(rx_include);
	 if(rx_exclude)	mj->SetExclude(rx_exclude);
	 mj->local_dir_base_len=local_dir_base_len;
	 mj->remote_dir_base_len=remote_dir_base_len;

	 dir_made=false;

	 break;
      }
      case(file->SYMLINK):
	 if(how!=0)
	 {
	    to_transfer->next();
	    break;
	 }
	 if(flags&REVERSE)
	 {
	    // can't create symlink remotely
	    to_transfer->next();
	    break;
	 }

#ifdef HAVE_LSTAT
	 if(file->defined&file->SYMLINK)
	 {
	    struct stat st;
	    if(lstat(local_name,&st)!=-1)
	    {
	       mod_symlinks++;
	       remove(local_name);
	    }
	    else
	    {
	       new_symlinks++;
	    }
	    res=symlink(file->symlink,local_name);
	    if(res==-1)
	       perror(local_name);
	 }
#endif /* LSTAT */
	 to_transfer->next();
	 break;
      }
   }
   else
   {
      // no info on type -- try to get
      goto try_get;
   }
}

void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
{
   source->Count(NULL,&tot_files,&tot_symlinks,&tot_files);

   to_rm=new FileSet(dest);
   to_rm->SubtractAny(source);

   same=new FileSet(source);
   to_transfer=new FileSet(source);
   to_transfer->SubtractSame(dest,flags&ONLY_NEWER,prec);
   same->SubtractAny(to_transfer);
}
int   MirrorJob::Do()
{
   char	 **glob_res;
   int	 res;
   int	 i;
   int	 m=STALL;

   switch(state)
   {
   case(DONE):
      return STALL;

   case(INITIAL_STATE):
      if(create_remote_dir)
      {
	 create_remote_dir=false;
	 session->Mkdir(remote_dir,true);
	 state=MAKE_REMOTE_DIR;
	 return MOVED;
      }
      session->Chdir(remote_dir);
      state=CHANGING_REMOTE_DIR;
      return MOVED;

   case(MAKE_REMOTE_DIR):
      res=session->Done();
      if(res==Ftp::IN_PROGRESS)
	 return STALL;
      if(res<0)
      {
	 eprintf("mirror: remote mkdir(%s): %s\n",remote_dir,session->StrError(res));
// 	 exit_code=1;
	 state=DONE;
	 session->Close();
	 return MOVED;
      }
      session->Close();
      state=INITIAL_STATE;
      return MOVED;

   case(CHANGING_REMOTE_DIR):
      res=session->Done();
      if(res==Ftp::IN_PROGRESS)
	 return STALL;
      if(res<0)
      {
	 eprintf("mirror: %s\n",session->StrError(res));
	 state=DONE;
	 session->Close();
	 return MOVED;
      }
      session->Close();

      if(remote_dir_base_len==0)
      {
	 xfree(remote_dir);
	 remote_dir=xstrdup(session->GetCwd());
      	 remote_dir_base_len=strlen(dir_file(remote_dir,""));
      }

      glob=new RemoteGlob(session,"",Ftp::LONG_LIST);
      state=GETTING_LONG_LIST;
      return MOVED;

   case(GETTING_LONG_LIST):
      if(!glob->Done())
	 return STALL;
      if(glob->Error())
      {
	 eprintf("mirror LIST: %s\n",session->StrError(glob->ErrorCode()));
	 state=DONE;
      	 delete glob;
	 glob=0;
// 	 exit_code=1;
	 return MOVED;
      }
      glob_res=glob->GetResult();
      remote_set=parse_listing(glob_res);
      delete glob;

      glob=new RemoteGlob(session,"",Ftp::LIST);
      state=GETTING_SHORT_LIST;
      return MOVED;

   case(GETTING_SHORT_LIST):
      if(!glob->Done())
	 return STALL;
      if(glob->Error())
      {
	 eprintf("mirror NLST: %s\n",session->StrError(glob->ErrorCode()));
	 state=DONE;
      	 delete glob;
	 glob=0;
// 	 exit_code=1;
	 return MOVED;
      }
      glob_res=glob->GetResult();
      remote_set->Merge(glob_res);
      delete glob;
      glob=0;

      remote_set->Exclude(remote_dir+remote_dir_base_len,
	       rx_exclude?&rxc_exclude:0,rx_include?&rxc_include:0);

      state=GETTING_INFO;
      remote_set->rewind();
      return MOVED;

   case(GETTING_INFO):
      if(session->IsClosed())
      {
	 get_info=(Ftp::fileinfo*)xmalloc(sizeof(*get_info)*remote_set->get_fnum());
	 cnt=0;
	 for(;;)
	 {
	    file=remote_set->curr();
	    if(file==0)
	       break;

	    if((file->defined & file->TYPE &&
	       (file->filetype==file->SYMLINK || file->filetype==file->DIRECTORY))
	    || ((file->defined&(file->DATE|file->SIZE))==(file->DATE|file->SIZE)))
	    {
	       remote_set->next();
	       continue;
	    }
	    get_info[cnt].get_size=!(file->defined & file->SIZE);
	    if(!get_info[cnt].get_size)
	       get_info[cnt].size=-1;
	    get_info[cnt].get_time=!(file->defined & file->DATE);
	    if(!get_info[cnt].get_time)
	       get_info[cnt].time=(time_t)-1;
	    get_info[cnt].file=file->name;
	    cnt++;
	    remote_set->next();
	 }
	 if(cnt)
	 {
	    session->GetInfoArray(get_info,cnt);
      	 }
	 else
	    goto info_done;
      }
      res=session->Done();
      if(res==Ftp::DO_AGAIN)
	 return STALL;
      if(res==Ftp::IN_PROGRESS)
	 return STALL;
      assert(res==Ftp::OK);
      session->Close();

      for(i=0; i<cnt; i++)
      {
	 if(get_info[i].time!=(time_t)-1)
	    remote_set->SetDate(get_info[i].file,get_info[i].time);
	 if(get_info[i].size!=-1)
	    remote_set->SetSize(get_info[i].file,get_info[i].size);
      }

   info_done:
      xfree(get_info);
      get_info=0;

      local_set=local_files(local_dir);

      local_set->Exclude(local_dir+local_dir_base_len,
	       rx_exclude?&rxc_exclude:0,rx_include?&rxc_include:0);

      if(flags&REVERSE)
	 InitSets(local_set,remote_set);
      else
	 InitSets(remote_set,local_set);

      to_transfer->rewind();
      waiting=0;
      state=WAITING_FOR_SUBGET;
      return MOVED;

   case(WAITING_FOR_RM_BEFORE_PUT):
   {
      if(waiting)
      {
	 if(!waiting->Done())
	    return STALL;
	 delete waiting;
      }
      ArgV *args=new ArgV("put");
      args->Append(dir_file(local_dir,file->name));
      args->Append(file->name);
      waiting=new PutJob(Clone(),args);
      waiting->parent=this;
      waiting->cmdline=(char*)xmalloc(6+strlen(file->name));
      sprintf(waiting->cmdline,"\\put %s",file->name);
      state=WAITING_FOR_SUBGET;
      return MOVED;
   }

   case(WAITING_FOR_SUBGET):
      if(waiting && waiting->Done())
      {
	 to_transfer->next();
	 delete waiting;
	 waiting=0;
      }
      while(!waiting && state==WAITING_FOR_SUBGET)
      {
	 file=to_transfer->curr();
      	 if(!file)
	 {
	    to_transfer->rewind();
	    state=WAITING_FOR_SUBMIRROR;
	    return MOVED;
	 }
	 HandleFile(0);
	 m=MOVED;
      }
      return m;

   case(WAITING_FOR_MKDIR_BEFORE_SUBMIRROR):
      if(waiting)
      {
	 if(!waiting->Done())
	    return STALL;
	 if(waiting->ExitCode()==0)
	    dir_made=true;
	 delete waiting;
	 waiting=0;
	 if(!dir_made)
	    to_transfer->next();
      }
      state=WAITING_FOR_SUBMIRROR;
      return MOVED;

   case(WAITING_FOR_SUBMIRROR):
      if(waiting && waiting->Done())
      {
	 MirrorJob &mj=*(MirrorJob*)waiting; // we are sure it is a MirrorJob
	 tot_files+=mj.tot_files;
	 new_files+=mj.new_files;
	 mod_files+=mj.mod_files;
	 del_files+=mj.del_files;
	 tot_symlinks+=mj.tot_symlinks;
	 new_symlinks+=mj.new_symlinks;
	 mod_symlinks+=mj.mod_symlinks;
	 del_symlinks+=mj.del_symlinks;
	 dirs+=mj.dirs;
	 del_dirs+=mj.del_dirs;

	 to_transfer->next();
	 delete waiting;
	 waiting=0;
      }
      while(!waiting && state==WAITING_FOR_SUBMIRROR)
      {
	 file=to_transfer->curr();
      	 if(!file)
	 {
	    to_rm->Count(&del_dirs,&del_files,&del_symlinks,&del_files);
	    if(flags&DELETE)
	       to_rm->LocalRemove(local_dir);
	    if(!(flags&NO_PERMS))
	       to_transfer->LocalChmod(local_dir,flags&ALLOW_SUID?0:S_ISUID|S_ISGID);
	    to_transfer->LocalUtime(local_dir);
	    if(!(flags&NO_PERMS))
	       same->LocalChmod(local_dir,flags&ALLOW_SUID?0:S_ISUID|S_ISGID);
	    same->LocalUtime(local_dir); // the old mtime can differ up to prec
	    state=DONE;
	    return MOVED;
	 }
	 HandleFile(1);
	 m=MOVED;
      }
      return m;
   }
   /*NOTREACHED*/
   abort();
}

MirrorJob::MirrorJob(FileAccess *f,const char *new_local_dir,const char *new_remote_dir)
   : SessionJob(f)
{
   local_dir=xstrdup(new_local_dir);
   remote_dir=xstrdup(new_remote_dir);
   local_dir_base_len=strlen(dir_file(local_dir,""));
   remote_dir_base_len=0;

   to_transfer=to_rm=same=0;
   remote_set=local_set=0;
   file=0;
   glob=0;

   tot_files=new_files=mod_files=del_files=
   tot_symlinks=new_symlinks=mod_symlinks=del_symlinks=0;
   dirs=1; del_dirs=0;

   get_info=0;

   flags=0;

   rx_include=rx_exclude=0;

   prec=0;  // time must be exactly same by default

   if(strcmp(f->GetProto(),"ftp"))
   {
      eprintf("Sorry, mirror currently works with ftp only\n");
      state=DONE;
      return;
   }

   state=INITIAL_STATE;

   dir_made=false;
   create_remote_dir=false;
}

MirrorJob::~MirrorJob()
{
   xfree(get_info);
   xfree(local_dir);
   xfree(remote_dir);
   if(local_set)
      delete local_set;
   if(remote_set)
      delete remote_set;
   if(to_transfer)
      delete to_transfer;
   if(to_rm)
      delete to_rm;
   if(same)
      delete same;
   // don't delete this->file -- it is a reference
   if(glob)
      delete glob;
   if(rx_include)
   {
      free(rx_include);
      regfree(&rxc_include);
   }
   if(rx_exclude)
   {
      free(rx_exclude);
      regfree(&rxc_exclude);
   }
}

int MirrorJob::SetInclude(const char *s)
{
   if(rx_include)
   {
      free(rx_include);
      rx_include=0;
      regfree(&rxc_include);
   }
   int res=regcomp(&rxc_include,s,REG_NOSUB|REG_EXTENDED);
   if(res!=0)
   {
      char err[1024];
      regerror(res,&rxc_include,err,sizeof(err));
      eprintf("mirror: --include - %s\n",err);
      return -1;
   }
   rx_include=xstrdup(s);
   return 0;
}
int MirrorJob::SetExclude(const char *s)
{
   if(rx_exclude)
   {
      free(rx_exclude);
      rx_exclude=0;
      regfree(&rxc_exclude);
   }
   int res=regcomp(&rxc_exclude,s,REG_NOSUB|REG_EXTENDED);
   if(res!=0)
   {
      char err[1024];
      regerror(res,&rxc_exclude,err,sizeof(err));
      eprintf("mirror: --exclude - %s\n",err);
      return -1;
   }
   rx_exclude=xstrdup(s);
   return 0;
}
