/*
 * lftp and utils
 *
 * Copyright (c) 1996-1998 by Alexander V. Lukyanov (lav@yars.free.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "bookmark.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>

#define super KeyValueDB

Bookmark::Bookmark()
{
   char *home=getenv("HOME");
   if(home==0)
      home=".";
   int home_len=strlen(home);
   const char *add="/.lftp/bookmarks";
   bm_file=(char*)xmalloc(home_len+strlen(add)+1);
   strcat(strcpy(bm_file,home),add);

   bm_fd=-1;
   stamp=(time_t)-1;

   Load();
}

Bookmark::~Bookmark()
{
   if(bm_fd!=-1)
      close(bm_fd);
   xfree(bm_file);
}

void Bookmark::Refresh()
{
   struct stat st;
   if(stat(bm_file,&st)==-1)
      return;
   if(st.st_mtime==stamp)
      return;
   Load();
}

void Bookmark::Lock(int type)
{
   struct flock	lk;
   lk.l_type=type;
   lk.l_whence=0;
   lk.l_start=0;
   lk.l_len=0;
   fcntl(bm_fd,F_SETLKW,&lk);
}

void Bookmark::Load()
{
   super::Empty();
   bm_fd=open(bm_file,O_RDONLY);
   if(bm_fd==-1)
      return;
   Lock(F_RDLCK);
   struct stat st;
   fstat(bm_fd,&st);
   stamp=st.st_mtime;
   super::Read(bm_fd);
   close(bm_fd);
   bm_fd=-1;
}

const char *Bookmark::Lookup(const char *key)
{
   Refresh();
   return super::Lookup(key);
}

#undef super

int KeyValueDB::Read(int fd)
{
   int key_size=16;
   char *key=(char*)xmalloc(key_size);
   int value_size=16;
   char *value=(char*)xmalloc(value_size);

   int c;

   FILE *f=fdopen(fd,"r");

   for(;;)
   {
      c=getc(f);

      // skip leading space
      while(c!=EOF && (c==' ' || c=='\t'))
	 c=getc(f);

      if(c==EOF)
	 break;
      if(c=='\n')
	 continue;   // next line

      char *store=key;

      for(;;)
      {
	 if(store>=key+key_size-1)
	 {
	    int shift=store-key;
	    key=(char*)xrealloc(key,key_size*=2);
	    store=key+shift;
	 }
	 *store++=c;

	 c=getc(f);
	 if(c==EOF)
	    break;
	 if(c==' ' || c=='\t' || c=='\n')
	    break;
      }
      *store=0;

      if(c==EOF || c=='\n' || store==key)
	 break;	// invalid line

      // skip space in mid
      while(c!=EOF && (c==' ' || c=='\t'))
	 c=getc(f);

      if(c==EOF || c=='\n')
	 break;

      store=value;

      for(;;)
      {
	 if(store>=value+value_size-1)
	 {
	    int shift=store-value;
	    value=(char*)xrealloc(value,value_size*=2);
	    store=value+shift;
	 }
	 *store++=c;

	 c=getc(f);
	 if(c==EOF)
	    break;
	 if(c=='\n')
	    break;
      }
      *store=0;

      Add(key,value);

      if(c==EOF)
	 break;
   }
   fclose(f);
   xfree(key);
   xfree(value);
   return 0;
}

void KeyValueDB::Add(const char *key,const char *value)
{
   chain=new Pair(key,value,chain);
}

const char *KeyValueDB::Lookup(const char *key)
{
   for(Pair *p=chain; p; p=p->next)
   {
      if(p->KeyCompare(key)==0)
	 return p->value;
   }
   return 0;
}
