/*
 * lftp and utils
 *
 * Copyright (c) 1996-1998 by Alexander V. Lukyanov (lav@yars.free.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef BOOKMARK_H
#define BOOKMARK_H

#include <sys/types.h>
#include "xmalloc.h"

class KeyValueDB
{
   class Pair
   {
   public:
      char *key;
      char *value;
      Pair *next;
      Pair(const char *k,const char *v,Pair *n)
	 {
	    key=xstrdup(k);
	    value=xstrdup(v);
	    next=n;
	 }
      ~Pair()
	 {
	    xfree(key);
	    xfree(value);
	 }
      int KeyCompare(const char *s)
	 {
	    return strcmp(s,key);
	 }
   };

   Pair *chain;

   void Purge(Pair **p)
      {
	 Pair *to_free=*p;
	 *p=to_free->next;
	 delete to_free;
      }
public:
   void Add(const char *id,const char *value);
   void Remove(const char *id);
   const char *Lookup(const char *id);
   void Empty()
      {
	 while(chain)
	    Purge(&chain);
      }
   int Write(int fd);
   int Read(int fd);

   KeyValueDB()
      {
	 chain=0;
      }
   ~KeyValueDB()
      {
	 Empty();
      }
};

class Bookmark : public KeyValueDB
{
   char *bm_file;
/*   char *bm_html_file;*/
   int bm_fd;
   time_t stamp;

   void Save();
   void Load();
   void Refresh();
   void Lock(int type);
public:
   void Add(const char *id,const char *value);
   void Remove(const char *id);
   const char *Lookup(const char *id);

   Bookmark();
   ~Bookmark();
};

#endif //BOOKMARK_H
